@@ -325,6 +325,22 @@ return {
         )
       end)
     },
+    group 'clone' {
+      test('function exists', function ()
+        assert.are_equal(type(Inlines({}).clone), 'function')
+      end),
+      test('clones the list', function ()
+        local inlns = Inlines{'Hello,', Space(), 'World!'}
+        assert.are_same(inlns, inlns:clone())
+      end),
+      test('deep-clones the list', function ()
+        local inlns = Inlines{Str 'Hello,', Space(), Str 'World!'}
+        local copy = inlns:clone()
+        copy[1].text = 'Bonjour,'
+        assert.are_same(Inlines{Str 'Bonjour,', Space(), Str 'World!'}, copy)
+        assert.are_same(Inlines{Str 'Hello,', Space(), Str 'World!'}, inlns)
+      end)
+    },
     group 'tostring' {
       test('works on an empty list', function ()
         assert.are_equal(
@@ -441,5 +457,46 @@ return {
         table.concat(names, ', ')
       )
     end),
+  },
+
+  group 'marshalling' {
+    test('bare strings become Str values', function ()
+      assert.are_equal(Inlines{'a'}, Inlines{Str 'a'})
+    end),
+
+    group '__toinline metamethod' {
+      test('metamethod __toinline is called when available', function ()
+        local function toinline (t)
+          return Code(t.code, {id = t.id, class = t.class})
+        end
+        local my_code = setmetatable(
+          {code = 'open access', id='opn'},
+          {__toinline = toinline}
+        )
+        assert.are_equal(
+          Inlines{Code('open access', {'opn'})},
+          Inlines{my_code}
+        )
+      end),
+
+      test("metafield is ignored if it's not a function", function ()
+        local bad_inline = setmetatable({'a'}, {__toinline = true})
+        assert.are_equal(
+          Inlines({'a'}),
+          Inlines(bad_inline)
+        )
+      end),
+
+      test("non-Inline return values are ignored", function ()
+        local function toinline ()
+          return "not an inline"
+        end
+        local bad_inline = setmetatable({'a'}, {__toinline = toinline})
+        assert.are_equal(
+          Inlines({'a'}),
+          Inlines(bad_inline)
+        )
+      end),
+    }
   }
 }
